@@ -6,8 +6,10 @@ use 5.008001;
 use strict;
 use warnings;
 use Carp;
+use Getopt::Long qw(:config bundling);
 use JSON;
 use Text::CSV;
+use Apache::Sling;
 use Apache::Sling::GroupUtil;
 use Apache::Sling::Print;
 use Apache::Sling::Request;
@@ -16,9 +18,9 @@ require Exporter;
 
 use base qw(Exporter);
 
-our @EXPORT_OK = ();
+our @EXPORT_OK = qw(command_line);
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 #{{{sub new
 
@@ -141,24 +143,6 @@ sub add_from_file {
 
 #}}}
 
-#{{{sub del
-sub del {
-    my ( $group, $act_on_group ) = @_;
-    my $res = Apache::Sling::Request::request(
-        \$group,
-        Apache::Sling::GroupUtil::delete_setup(
-            $group->{'BaseURL'}, $act_on_group
-        )
-    );
-    my $success = Apache::Sling::GroupUtil::delete_eval($res);
-    my $message = "Group: \"$act_on_group\" ";
-    $message .= ( $success ? 'deleted!' : 'was not deleted!' );
-    $group->set_results( "$message", $res );
-    return $success;
-}
-
-#}}}
-
 #{{{sub check_exists
 sub check_exists {
     my ( $group, $act_on_group ) = @_;
@@ -177,183 +161,221 @@ sub check_exists {
 
 #}}}
 
-#{{{sub member_add
-sub member_add {
-    my ( $group, $act_on_group, $add_member ) = @_;
+#{{{ sub command_line
+sub command_line {
+    my @ARGV = @_;
+
+    #options parsing
+    my $sling  = Apache::Sling->new;
+    my $config = config($sling);
+
+    GetOptions(
+        $config,      'auth=s',     'help|?',       'log|L=s',
+        'man|M',      'pass|p=s',   'threads|t=s',  'url|U=s',
+        'user|u=s',   'verbose|v+', 'add|a=s',      'additions|A=s',
+        'delete|d=s', 'exists|e=s', 'property|P=s', 'view|V=s'
+    ) or help();
+
+    if ( $sling->{'Help'} ) { help(); }
+    if ( $sling->{'Man'} )  { man(); }
+
+    return run( $sling, $config );
+}
+
+#}}}
+
+#{{{sub config
+
+sub config {
+    my ($sling) = @_;
+    my $additions;
+    my $add;
+    my $delete;
+    my $exists;
+    my @property;
+    my $view;
+
+    my %group_config = (
+        'auth'      => \$sling->{'Auth'},
+        'help'      => \$sling->{'Help'},
+        'log'       => \$sling->{'Log'},
+        'man'       => \$sling->{'Man'},
+        'pass'      => \$sling->{'Pass'},
+        'threads'   => \$sling->{'Threads'},
+        'url'       => \$sling->{'URL'},
+        'user'      => \$sling->{'User'},
+        'verbose'   => \$sling->{'Verbose'},
+        'add'       => \$add,
+        'additions' => \$additions,
+        'delete'    => \$delete,
+        'exists'    => \$exists,
+        'property'  => \@property,
+        'view'      => \$view
+    );
+
+    return \%group_config;
+}
+
+#}}}
+
+#{{{sub del
+sub del {
+    my ( $group, $act_on_group ) = @_;
     my $res = Apache::Sling::Request::request(
         \$group,
-        Apache::Sling::GroupUtil::member_add_setup(
-            $group->{'BaseURL'}, $act_on_group, $add_member
+        Apache::Sling::GroupUtil::delete_setup(
+            $group->{'BaseURL'}, $act_on_group
         )
     );
-    my $success = Apache::Sling::GroupUtil::member_add_eval($res);
-    my $message = "\"$add_member\" ";
-    $message .= ( $success ? 'added' : 'was not added' );
-    $message .= " to group \"$act_on_group\"!";
+    my $success = Apache::Sling::GroupUtil::delete_eval($res);
+    my $message = "Group: \"$act_on_group\" ";
+    $message .= ( $success ? 'deleted!' : 'was not deleted!' );
     $group->set_results( "$message", $res );
     return $success;
 }
 
 #}}}
 
-#{{{sub member_add_from_file
-sub member_add_from_file {
-    my ( $group, $file, $fork_id, $number_of_forks ) = @_;
-    $fork_id         = defined $fork_id         ? $fork_id         : 0;
-    $number_of_forks = defined $number_of_forks ? $number_of_forks : 1;
-    my $csv               = Text::CSV->new();
-    my $count             = 0;
-    my $number_of_columns = 0;
-    my @column_headings;
-    if ( !defined $file ) {
-        croak 'File to upload from not defined';
-    }
-    if ( open my ($input), '<', $file ) {
-        while (<$input>) {
-            if ( $count++ == 0 ) {
+#{{{ sub help
+sub help {
 
-                # Parse file column headings first to determine field names:
-                if ( $csv->parse($_) ) {
-                    @column_headings = $csv->fields();
+    print <<"EOF";
+Usage: perl $0 [-OPTIONS [-MORE_OPTIONS]] [--] [PROGRAM_ARG1 ...]
+The following options are accepted:
 
-                    # First field must be group:
-                    if ( $column_headings[0] !~ /^[Gg][Rr][Oo][Uu][Pp]$/msx ) {
-                        croak 'First CSV column must be the group ID, '
-                          . 'column heading must be "group". '
-                          . 'Found: "'
-                          . $column_headings[0] . "\".\n";
-                    }
+ --additions or -A (file)          - file containing list of groups to be added.
+ --add or -a (actOnGroup)          - add specified group.
+ --auth (type)                     - Specify auth type. If ommitted, default is used.
+ --delete or -d (actOnGroup)       - delete specified group.
+ --exists or -e (actOnGroup)       - check whether specified group exists.
+ --help or -?                      - view the script synopsis and options.
+ --log or -L (log)                 - Log script output to specified log file.
+ --man or -M                       - view the full script documentation.
+ --pass or -p (password)           - Password of user performing actions.
+ --property or -P (property=value) - Specify property to set on group.
+ --threads or -t (threads)         - Used with -A, defines number of parallel
+                                     processes to have running through file.
+ --url or -U (URL)                 - URL for system being tested against.
+ --user or -u (username)           - Name of user to perform any actions as.
+ --verbose or -v or -vv or -vvv    - Increase verbosity of output.
+ --view or -V (actOnGroup)         - view details for specified group in json format.
 
-                    # Second field must be user:
-                    if ( $column_headings[1] !~ /^[Uu][Ss][Ee][Rr]$/msx ) {
-                        croak 'Second CSV column must be the user ID, '
-                          . 'column heading must be "user". '
-                          . 'Found: "'
-                          . $column_headings[1] . "\".\n";
-                    }
-                    $number_of_columns = @column_headings;
-                }
-                else {
-                    croak 'CSV broken, failed to parse line: '
-                      . $csv->error_input;
-                }
-            }
-            elsif ( $fork_id == ( $count++ % $number_of_forks ) ) {
-                if ( $csv->parse($_) ) {
-                    my @columns      = $csv->fields();
-                    my $columns_size = @columns;
+Options may be merged together. -- stops processing of options.
+Space is not required between options and their arguments.
+For full details run: perl $0 --man
+EOF
 
-           # Check row has same number of columns as there were column headings:
-                    if ( $columns_size != $number_of_columns ) {
-                        croak
-"Found \"$columns_size\" columns. There should have been \"$number_of_columns\".\n"
-                          . "Row contents was: $_";
-                    }
-                    my $act_on_group = $columns[0];
-                    my $add_member   = $columns[1];
-                    $group->member_add( $act_on_group, $add_member );
-                    Apache::Sling::Print::print_result($group);
-                }
-                else {
-                    croak 'CSV broken, failed to parse line: '
-                      . $csv->error_input;
-                }
-            }
-        }
-        close $input or croak q{Problem closing input!};
-    }
-    else {
-        croak "Problem opening file: '$file'";
-    }
     return 1;
 }
 
 #}}}
 
-#{{{sub member_delete
-sub member_delete {
-    my ( $group, $act_on_group, $delete_member ) = @_;
-    my $res = Apache::Sling::Request::request(
-        \$group,
-        Apache::Sling::GroupUtil::member_delete_setup(
-            $group->{'BaseURL'}, $act_on_group, $delete_member
-        )
-    );
-    my $success = Apache::Sling::GroupUtil::member_delete_eval($res);
-    my $message = "\"$delete_member\" ";
-    $message .= ( $success ? 'deleted' : 'was not deleted' );
-    $message .= " from group \"$act_on_group\"!";
-    $group->set_results( "$message", $res );
-    return $success;
+#{{{ sub man
+sub man {
+
+    print <<'EOF';
+group perl script. Provides a means of managing groups in sling from the
+command line. The script also acts as a reference implementation for the Group
+perl library.
+
+EOF
+
+    help();
+
+    print <<"EOF";
+Example Usage
+
+* Authenticate and add a group with id g-test:
+
+ perl group.pl -U http://localhost:8080 -u admin -p admin -a g-test
+
+* Authenticate and check whether group with id g-test exists:
+
+ perl group.pl -U http://localhost:8080 -u admin -p admin -a g-test
+
+* Authenticate and view details for group with id g-test:
+
+ perl group.pl -U http://localhost:8080 -u admin -p admin -V g-test
+
+* Authenticate and delete group with id g-test:
+
+ perl group.pl -U http://localhost:8080 -u admin -p admin -d g-test
+
+* Authenticate and add a group with id g-test and property p1=v1:
+
+ perl group.pl -U http://localhost:8080 -u admin -p admin -a g-test -P p1=v1
+
+EOF
+
+    return 1;
 }
 
 #}}}
 
-#{{{sub member_exists
-sub member_exists {
-    my ( $group, $act_on_group, $exists_member ) = @_;
-    my $res = Apache::Sling::Request::request(
-        \$group,
-        Apache::Sling::GroupUtil::view_setup(
-            $group->{'BaseURL'}, $act_on_group
-        )
-    );
-    my $success = Apache::Sling::GroupUtil::view_eval($res);
-    my $message;
-    if ($success) {
-        my $group_info = from_json( ${$res}->content );
-        my $is_member  = 0;
-        foreach my $member ( @{ $group_info->{'members'} } ) {
-            if (   $member eq "/system/userManager/user/$exists_member"
-                || $member eq "/system/userManager/group/$exists_member"
-                || $member eq "$exists_member" )
-            {
-                $is_member = 1;
-                last;
+#{{{sub run
+sub run {
+    my ( $sling, $config ) = @_;
+    if ( !defined $config ) {
+        croak 'No group config supplied!';
+    }
+    $sling->check_forks;
+    my $authn =
+      defined $sling->{'Authn'}
+      ? ${ $sling->{'Authn'} }
+      : new Apache::Sling::Authn( \$sling );
+
+    my $success = 1;
+
+    if ( defined ${ $config->{'additions'} } ) {
+        my $message =
+          "Adding groups from file \"" . ${ $config->{'additions'} } . "\":\n";
+        Apache::Sling::Print::print_with_lock( "$message", $sling->{'Log'} );
+        my @childs = ();
+        for my $i ( 0 .. $sling->{'Threads'} ) {
+            my $pid = fork;
+            if ($pid) { push @childs, $pid; }    # parent
+            elsif ( $pid == 0 ) {                # child
+                    # Create a new separate user agent per fork in order to
+                    # ensure cookie stores are separate, then log the user in:
+                $authn->{'LWP'} = $authn->user_agent( $sling->{'Referer'} );
+                $authn->login_user();
+                my $group =
+                  new Apache::Sling::Group( \$authn, $sling->{'Verbose'},
+                    $sling->{'Log'} );
+                $group->add_from_file( ${ $config->{'additions'} },
+                    $i, $sling->{'Threads'} );
+                exit 0;
+            }
+            else {
+                croak "Could not fork $i!";
             }
         }
-        $success = $is_member;
-        $message =
-            "\"$exists_member\" is "
-          . ( $is_member ? q{} : 'not ' )
-          . "in group \"$act_on_group\"";
+        foreach (@childs) { waitpid $_, 0; }
     }
     else {
-        $message = "Problem viewing group: \"$act_on_group\"";
-    }
-    $group->set_results( "$message", $res );
-    return $success;
-}
-
-#}}}
-
-#{{{sub member_view
-sub member_view {
-    my ( $group, $act_on_group ) = @_;
-    my $res = Apache::Sling::Request::request(
-        \$group,
-        Apache::Sling::GroupUtil::view_setup(
-            $group->{'BaseURL'}, $act_on_group
-        )
-    );
-    my $success = Apache::Sling::GroupUtil::view_eval($res);
-    my $message;
-    if ($success) {
-        my $group_info     = from_json( ${$res}->content );
-        my $number_members = @{ $group_info->{'members'} };
-        my $members = "$number_members result(s) for group \"$act_on_group\":";
-        foreach my $member ( @{ $group_info->{'members'} } ) {
-            $members .= "\n$member";
+        $authn->login_user();
+        my $group =
+          new Apache::Sling::Group( \$authn, $sling->{'Verbose'},
+            $sling->{'Log'} );
+        if ( defined ${ $config->{'exists'} } ) {
+            $success = $group->check_exists( ${ $config->{'exists'} } );
         }
-        $message = "$members";
-        $success = $number_members;
+        elsif ( defined ${ $config->{'add'} } ) {
+            $success =
+              $group->add( ${ $config->{'add'} }, $config->{'property'} );
+        }
+        elsif ( defined ${ $config->{'delete'} } ) {
+            $success = $group->del( ${ $config->{'delete'} } );
+        }
+        elsif ( defined ${ $config->{'view'} } ) {
+            $success = $group->view( ${ $config->{'view'} } );
+        }
+        else {
+            help();
+            return 1;
+        }
+        Apache::Sling::Print::print_result($group);
     }
-    else {
-
-        # HTTP request did not complete successfully!
-        $message = "Problem viewing group: \"$act_on_group\"";
-    }
-    $group->set_results( "$message", $res );
     return $success;
 }
 
@@ -410,6 +432,10 @@ Add a new group to the system.
 
 Add new groups to the system based on definitions in a file.
 
+=head2 config
+
+Fetch hash of group configuration.
+
 =head2 del
 
 Delete a user.
@@ -418,25 +444,9 @@ Delete a user.
 
 Check whether a group exists.
 
-=head2 member_add
+=head2 run
 
-Add a member to a group.
-
-=head2 member_add_from_file
-
-Add members to groups based on entries in a file.
-
-=head2 member_delete
-
-Delete member from a group.
-
-=head2 member_exists
-
-Check whether a member exists in a group.
-
-=head2 member_view
-
-View members of a group.
+Run group related actions.
 
 =head2 view
 
